var margin = { left: 100, right: 10, top: 10, bottom: 150 };

var width = 900 - margin.left - margin.right,
  height = 600 - margin.top - margin.bottom;

var flag = true;

var t = d3.transition().duration(750);

var g = d3
  .select(".chart")
  .append("svg")
  .attr("width", width + margin.left + margin.right)
  .attr("height", height + margin.top + margin.bottom)
  .append("g")
  .attr("transform", "translate(" + margin.left + ", " + margin.top + ")");

var xScale = d3
  .scaleBand()
  .range([0, width])
  .paddingInner(0.3)
  .paddingOuter(0.3);

var yScale = d3.scaleLinear().range([height, 0]);

// x-Achse
var xAxisGroup = g
  .append("g")
  .attr("class", "x axis")
  .attr("transform", "translate(0, " + height + ")");

// x-Achsenbeschriftung
g.append("text")
  .attr("class", "x axis-label")
  .attr("x", width / 2)
  .attr("y", height + 140)
  .attr("font-size", "20px")
  .attr("fill", "#ccc")
  .attr("text-anchor", "middle")
  .text("Gebäudename");

// y-Achse
var yAxisGroup = g.append("g").attr("class", "y axis");

// y-Achsenbeschriftung
var yLabel = g
  .append("text")
  .attr("class", "y axis-label")
  .attr("x", -(height / 2))
  .attr("y", -60)
  .attr("font-size", "20px")
  .attr("text-anchor", "middle")
  .attr("fill", "#ccc")
  .attr("transform", "rotate(-90)");

d3.json("data/buildings.json").then(function(data) {
  // Clean data
  data.forEach(function(d) {
    d.height = +d.height;
    d.visitors = +d.visitors;
  });

  d3.select("#changeData").on("click", function() {
    flag = !flag;
    var newData = flag ? data : data.slice(1);
    update(newData);
  });

  // Run the vis for the first time
  update(data);
});

function update(data) {
  var value = flag ? "height" : "visitors";

  xScale.domain(
    data.map(function(d) {
      return d.name;
    })
  );

  yScale.domain([
    0,
    d3.max(data, function(d) {
      return d[value];
    })
  ]);

  // xAxis
  var xAxisGenerator = d3.axisBottom(xScale);
  xAxisGroup
    .transition(t)
    .call(xAxisGenerator)
    .selectAll("text")
    .attr("y", 10)
    .attr("x", -5)
    .attr("text-anchor", "end")
    .attr("transform", "rotate(-40)");

  // yAxis
  var yAxisGenerator = d3.axisLeft(yScale);
  yAxisGroup.transition(t).call(yAxisGenerator);

  // Data Join
  var rects = g.selectAll("rect").data(data, function(d) {
    return d.name;
  });

  // Exit
  rects
    .exit()
    .transition(t)
    .attr("y", yScale(0))
    .attr("height", 0)
    .remove();

  // Enter
  rects
    .enter()
    .append("rect")
    .attr("fill", "grey")
    .attr("x", function(d) {
      return xScale(d.name);
    })
    .attr("y", yScale(0))
    .attr("height", 0)
    .attr("width", xScale.bandwidth())
    // Merge führt das Update aus
    .merge(rects)
    .transition(t)
    .attr("x", function(d) {
      return xScale(d.name);
    })
    .attr("y", function(d) {
      return yScale(d[value]);
    })
    .attr("height", function(d) {
      return height - yScale(d[value]);
    });

  var label = flag ? "Höhe (m)" : "Besucher";
  yLabel.text(label);
}
